@@ -417,16 +417,16 @@ void pdl_dump_flags(int flags, int nspac)
 {
 	int i;
 	int len, found, sz;
-	int flagval[13] = {PDL_ALLOCATED,PDL_PARENTDATACHANGED,
+	int flagval[14] = {PDL_ALLOCATED,PDL_PARENTDATACHANGED,
 			 PDL_PARENTDIMSCHANGED,PDL_PARENTREPRCHANGED,
 			 PDL_DATAFLOW_F,PDL_DATAFLOW_B,PDL_NOMYDIMS,
 			 PDL_OPT_VAFFTRANSOK,PDL_INPLACE,PDL_DESTROYING,
-			 PDL_DONTTOUCHDATA, PDL_MYDIMS_TRANS, 0};
-	char *flagchar[12] = {"ALLOCATED","PARENTDATACHANGED",
+			 PDL_DONTTOUCHDATA, PDL_MYDIMS_TRANS, PDL_HDRCPY, 0};
+	char *flagchar[13] = {"ALLOCATED","PARENTDATACHANGED",
 			 "PARENTDIMSCHANGED","PARENTREPRCHANGED",
 			 "DATAFLOW_F","DATAFLOW_B","NOMYDIMS",
 			 "OPT_VAFFTRANSOK","INPLACE","DESTROYING",
-			 "DONTTOUCHDATA","MYDIMS_TRANS"};
+			 "DONTTOUCHDATA","MYDIMS_TRANS", "HDRCPY"};
 	char *spaces = malloc(nspac+1); for(i=0; i<nspac; i++) spaces[i]=' ';
 	spaces[i] = '\0';
 
@@ -1096,8 +1096,12 @@ void pdl__ensure_trans(pdl_trans *trans,int what)
 			} else
 #endif
 #endif
-				if(!trans->pdls[j]) {return;} /* XXX!!! */
+                       {       if(!trans->pdls[j]) {return;} /* XXX!!! */
+                       PDLDEBUG_f(printf("not vaffine ok: %d\n",
+                                         trans->vtable->per_pdl_flags[j]));
+
 				pdl_make_physical(trans->pdls[j]);
+                       }
 		}
 		flag |= trans->pdls[j]->state & PDL_ANYCHANGED;
 	}
@@ -1309,7 +1313,8 @@ void pdl_make_physvaffine(pdl *it)
 	}
 	PDL_ENSURE_VAFFTRANS(it);
 	incsleft = malloc(sizeof(*incsleft)*it->ndims);
-	for(i=0; i<it->ndims; i++) {
+        PDLDEBUG_f(printf("vaff_malloc: got %d\n",incsleft));
+        for(i=0; i<it->ndims; i++) {
 		it->vafftrans->incs[i] = it->dimincs[i];
 	}
 
@@ -1395,7 +1400,8 @@ void pdl_make_physvaffine(pdl *it)
 	pdl_make_physical(current);
 
   mkphys_vaff_end:
-       free(incsleft);
+       PDLDEBUG_f(printf("vaff_malloc: %d\n",incsleft));
+       if (incsleft != NULL) free(incsleft);
 	PDLDEBUG_f(printf("Make_physvaffine_exit %d\n",it));
 
 }
