@@ -116,6 +116,7 @@ void pdl_initthreadstruct(int nobl,
 
 	thread->mag_nth = -1;
 	thread->mag_nthpdl = -1;
+        thread->mag_nthr = -1;
 
 	nids=0;
 	mx=0;
@@ -135,6 +136,7 @@ void pdl_initthreadstruct(int nobl,
 		  (nthr = pdl_magic_thread_nthreads(pdls[j],&nthrd))) {
 			thread->mag_nthpdl = j;
 			thread->mag_nth = nthrd - realdims[j];
+                       thread->mag_nthr = nthr;
 			if(thread->mag_nth < 0) {
 				die("Cannot magick non-threaded dims");
 			}
@@ -195,7 +197,7 @@ void pdl_initthreadstruct(int nobl,
 				if(thread->dims[nth] != 1) {
 					if(thread->dims[nth] !=
 						pdls[j]->dims[i+realdims[j]]) {
-						pdl_croak_param(info,j,"Mismatched Implicit thread dimension %d: should be %d, is %d",
+                                              pdl_croak_param(info,j,"Mismatched implicit thread dimension %d: should be %d, is %d\n\t",
 							i,
 							thread->dims[nth],
 							pdls[j]->dims[i+thread->realdims[j]]);
@@ -284,7 +286,9 @@ See the manual for why this is impossible");
 	if (!temp)
 	  for(i=0; i<thread->nimpl; i++)
 		thread->pdls[j]->dims[i+thread->realdims[j]] =
-			thread->dims[i];
+                       thread->dims[i] *
+                     ((i == thread->mag_nth && thread->mag_nthr > 0) ?
+                           thread->mag_nthr : 1);
 	thread->pdls[j]->threadids[0] = td + thread->realdims[j];
 	pdl_resize_defaultincs(thread->pdls[j]);
 	for(i=0; i<thread->nimpl; i++) {
@@ -355,13 +359,20 @@ calc_offs:
 	return stopdim+1;
 }
 
+/* prototype, defined in pdlcore.c */
+char *pdl_mess(const char *pat, va_list *args);
 void pdl_croak_param(pdl_errorinfo *info,int j, char *pat, ...)
 {
 	va_list args;
 	char *message; char *name;
 	static char mesgbuf[200];
+      static char argsbuf[256], *argb;
+      int i, k, l;
+      SV *msv;
+
 	va_start(args,pat);
-	message = mess(pat,&args);
+	/* was Perl_mess before; Perl_mess has changed between 5.00X and 5.6 */
+	message = pdl_mess(pat,&args);  /* barf dependence !!!! */
 	/* Now, croak() overwrites this string. make a copy */
 	strcpy(mesgbuf,message); message = mesgbuf;
 	va_end(args);
@@ -371,7 +382,37 @@ void pdl_croak_param(pdl_errorinfo *info,int j, char *pat, ...)
 		if(j >= info->nparamnames)
 			name = "ERROR: UNKNOWN PARAMETER";
 		else	name = info->paramnames[j];
-		croak("PDL: %s: Parameter '%s': %s\n",info->funcname,name,message);
+              for (i=0,argb=argsbuf,l=255;i<info->nparamnames && l;i++) {
+                /* Could improve method, but 256 chars should be
+                     enough anyway! */
+                k = strlen(info->paramnames[i]);
+                if (k < l-4) {
+                  memcpy(argb,info->paramnames[i],k);
+                  argb += k;
+                  *argb = ',';
+                  argb++;
+                  l -= k+1;
+                } else {
+                  *argb++ = '.';
+                  *argb++ = '.';
+                  *argb++ = '.';
+                  argb++;
+                  l = 0;
+                }
+              }
+              *--argb = '\0';
+/* this needs to be sorted: barf stuff ?? */
+#ifdef croak
+#define tcroak croak
+#define croak Perl_croak
+#endif
+              croak("PDL: %s(%s): Parameter '%s'\n%s\n",
+                    info->funcname,argsbuf,name,message);
+#ifdef tcroak
+#undef croak
+#define croak tcroak
+#undef tcroak
+#endif
 	}
 }
 
