@@ -6,9 +6,9 @@ PDL - Main loader of PDL default modules
 
 Loads the default set of modules associated
 with PDL, making the functions available in
-the current namespace. See also C<PDL::Lite>,
-C<PDL::LiteF> if start-up time becomes an
-issue.
+the current namespace. See also 
+L<PDL::Lite|PDL::Lite> or L<PDL::LiteF|PDL::LiteF> 
+if start-up time becomes an issue.
 
 =head1 SYNOPSIS
 
@@ -21,14 +21,12 @@ issue.
    use PDL::Slices;
    use PDL::Version;
    use PDL::IO::Misc;
-   use PDL::Graphics::PGPLOT;
 
 =cut
 
 
-# get the version: 
-use PDL::Version;
-$PDL::VERSION = $PDL::Version::VERSION;
+# set the version: 
+$PDL::VERSION = '2.1.1';
 
 # Main loader of standard PDL package
 
@@ -50,7 +48,6 @@ use PDL::Slices;
 # Load these for TPJ compatibility
 
 use PDL::IO::Misc;          # Misc IO (Ascii/FITS)
-use PDL::Graphics::PGPLOT;  # PGPLOT graphics
 
 EOD
 
@@ -59,6 +56,11 @@ die $@ if $@;
 }
 
 
+# Dummy Package PDL Statement. This is only needed so CPAN
+# properly recognizes the PDL package.
+package PDL;
+
+
 ;# Exit with OK status
 
 1;
