@@ -153,7 +153,7 @@ sub trim {
 
     $txt =~ s/^\s*//;
     $txt =~ s/\s*$//;
-    while( $txt =~ s/^\((.*)\)$/\1/ ) {}; # Strip BALANCED brackets
+    while( $txt =~ s/^\((.*)\)$/$1/ ) {}; # Strip BALANCED brackets
 
   }
   for (split "\n", $txt) {
@@ -168,7 +168,7 @@ sub trim {
 
 =head1 NAME
 
-  PDL::Doc -- support for PDL online documentation
+PDL::Doc - support for PDL online documentation
 
 =head1 SYNOPSIS
 
@@ -213,6 +213,7 @@ Individual functions in this section are introduced by
 
 
 =pod
+
 where signature is the argumentlist for a PP defined function
 as explained in L<PDL::PP>. Generally, PDL documentation is in
 valid POD format (see L<perlpod>) but uses the C<=for> directive
@@ -419,18 +420,42 @@ use PDL::Doc::Config;
 
 =head2 new
 
-  $onlinedc = new PDL::Doc ('file.pdl');
+  $onlinedc = new PDL::Doc ('file.pdl',[more files]);
 
 =cut
 
 sub new {
-  my ($type,$file) = @_;
-  $file = DefaultFile() unless $file;
+  my ($type,@files) = @_;
   my $this = bless {},$type;
-  $this->{File} = $file;
+  $this->{File} = [@files];
+  $this->{Scanned} = [];
+  $this->{Outfile} = $files[0];
   return $this;
 }
 
+=head2 addfiles
+
+add another file to the online database associated with this object.
+
+=cut
+
+sub addfiles {
+  my ($this,@files) = @_;
+  push @{$this->{File}}, @files;
+}
+
+=head2 outfile
+
+set the name of the output file for this online db
+
+=cut
+
+sub outfile {
+  my ($this,$file) = @_;
+  $this->{Outfile} = $file if defined $file;
+  return $this->{Outfile};
+}
+
 =head2 ensuredb
 
 Make sure that the database is slurped in
@@ -439,17 +464,20 @@ Make sure that the database is slurped in
 
 sub ensuredb {
   my ($this) = @_;
-  return $_[0]->{SYMS} if defined $_[0]->{SYMS};
-  open IN, $this->{File} or
-    barf "can't open database $this->{File}, scan docs first";
-  my ($plen,$txt);
-  while (read IN, $plen,2) {
-    my ($len) = unpack "S", $plen;
-    read IN, $txt, $len;
-    my ($sym, %hash) = split chr(0), $txt;
-    $this->{SYMS}->{$sym} = {%hash};
+  while (my $fi = pop @{$this->{File}}) {
+    open IN, $fi or
+      barf "can't open database $fi, scan docs first";
+    binmode IN;
+    my ($plen,$txt);
+    while (read IN, $plen,2) {
+      my ($len) = unpack "S", $plen;
+      read IN, $txt, $len;
+      my ($sym, %hash) = split chr(0), $txt;
+      $this->{SYMS}->{$sym} = {%hash};
+    }
+    close IN;
+    push @{$this->{Scanned}}, $fi;
   }
-  close IN;
   return $this->{SYMS};
 }
 
@@ -459,11 +487,12 @@ save the database (i.e., the hash of PDL symbols) to the file associated
 with this object.
 
 =cut
-
+ 
 sub savedb {
   my ($this) = @_;
   my $hash = $this->ensuredb();
-  open OUT, ">$this->{File}" or barf "can't write to symdb $this->{File}";
+  open OUT, ">$this->{Outfile}" or barf "can't write to symdb $this->{Outfile}";
+  binmode OUT;
   while (my ($key,$val) = each %$hash) {
     my $txt = "$key".chr(0).join(chr(0),%$val);
     print OUT pack("S",length($txt)).$txt;
@@ -574,20 +603,27 @@ sub scan {
   # XXXX convert to absolute path
   # my $outfile = '/tmp/'.basename($file).'.pod';
   my $outfile = new NullHandle;
+
+  # Handle RPM etc. case where we are building away from the final
+  # location. Alright it's a hack - KGB
+  my $file2 = $file;
+  $file2 =~ s/^$ENV{BUILDROOTPREFIX}// if $ENV{BUILDROOTPREFIX} ne "";
+
   my $parser = new PDL::PodParser;
   $parser->{verbose} = $verbose;
   $parser->parse_from_filehandle($infile,$outfile);
   $this->{SYMS} = {} unless defined $this->{SYMS};
   my $hash = $this->{SYMS};
   my @stats = stat $file;
-  $this->{FTIME}->{$file} = $stats[9]; # store last mod time
+  $this->{FTIME}->{$file2} = $stats[9]; # store last mod time
   # print "mtime of $file: $stats[9]\n";
   my $phash = $parser->{SYMHASH};
   my $n = 0;
   while (my ($key,$val) = each %$phash) {
     #print "adding '$key'\n";
     $n++;
-    $val->{File} = $file;
+
+    $val->{File} = $file2;
     $hash->{$key} = $val
     }
 
@@ -609,7 +645,7 @@ sub scan {
    }
    $does = 'Hmmm ????' if $does =~ /^\s*$/;
    my $type = ($file =~ /\.pod$/ ? 'Manual:' : 'Module:');
-   $hash->{$name} = {Ref=>"$type $does",File=>$file} if $name !~ /^\s*$/;
+   $hash->{$name} = {Ref=>"$type $does",File=>$file2} if $name !~ /^\s*$/;
    return $n;
 }
 
@@ -670,7 +706,10 @@ sub funcdocs_fromfile {
   barf "can't open file $file" unless $in;
   barf "can't open output handle" unless $out;
   getfuncdocs($func,$in,$out);
-  print $out "Docs from $file\n\n";
+
+  if (ref $out eq 'GLOB') {
+  	print $out "Docs from $file\n\n"; } else {
+	$out->print("Docs from $file\n\n"); }
 }
 
 sub extrdoc {
@@ -683,7 +722,7 @@ sub extrdoc {
 sub getfuncdocs {
   my ($func,$in,$out) = @_;
   my $parser = new PDL::Pod::Parser;
-  $parser->select("FUNCTIONS/$func(\\(.*\\))*\s*");
+  $parser->select("FUNCTIONS/$func(\\(.*\\))*\\s*");
   $parser->parse_from_filehandle($in,$out);
 }
 
@@ -696,11 +735,14 @@ discussions on the pdl-porters mailing list.
 
 =head1 AUTHOR
 
-Copyright 1997 Christian Soeller <csoelle@sghms.ac.uk> and Karl Glazebrook
-<kgb@aaoepp.aao.gov.au>
+Copyright 1997 Christian Soeller E<lt>c.soeller@auckland.ac.nzE<gt> 
+and Karl Glazebrook E<lt>kgb@aaoepp.aao.gov.auE<gt>
 All rights reserved. There is no warranty. You are allowed
 to redistribute this software / documentation under certain
 conditions. For details, see the file COPYING in the PDL
 distribution. If this file is separated from the PDL distribution,
 the copyright notice should be included in the file.
 
+=cut
+
+1;
